package org.hibernate.cache;

/**
 * Defines the contract for a cache region which will specifically be used to
 * store collection data.
 * <p/>
 * Impl note: Hibernate always deals with changes to collections which
 * (potentially) has its data in the L2 cache by removing that collection
 * data; in other words it never tries to update the cached state, thereby
 * allowing it to avoid a bunch of concurrency problems.
 *
 * @author Steve Ebersole
 */
public interface CollectionRegion extends AccessProtectableRegion {

	/**
	 * Iitialize the collection data region by letting it know some information
	 * regarding the type of data that will be stored there (specifically
	 * versioning information regarding its owner).
	 *
	 * @param source Metadata about the type of data this region will be
	 * storing.
	 */
	public void intialize(OptimisticCacheSource source);

	/**
	 * Retrieve the collection data for the given key from the underlying cache.
	 * Used during attempt to resolve collection state from the second level
	 * cache.
	 *
	 * @param key The cache key
	 * @return the cached object or <tt>null</tt>
	 * @throws CacheException Indicates a problem accessing the region.
	 */
	public Object get(Object key) throws CacheException;

	/**
	 * Called when the collection is read from the database and its state
	 * needs to be written to the L2 cache.
	 *
	 * @param key The cache key.
	 * @param value The data to be cached.
	 * @param version The owner's version, or null if owner not versioned.
	 * @throws CacheException Indicates a problem accessing the region.
	 */
	public void writeLoad(Object key, Object value, Object version) throws CacheException;

	/**
	 * Called when the collection has been modified and its new structure written
	 * to the database.  As mentioned above, Hibernate always invalidates
	 * L2 cache entries for modified collections.
	 *
	 * @param key The cache key.
	 * @throws CacheException Indicates a problem accessing the region.
	 */
	public void remove(Object key) throws CacheException;
}
