package org.hibernate.cache;

import java.util.Map;

/**
 * Defines a contract for accessing a particular named region within the 
 * underlying cache implementation.
 *
 * @author Steve Ebersole
 */
public interface Region {
	/**
	 * Retrieve the name of this region.
	 *
	 * @return The region name
	 */
	public String getName();

	/**
	 * Get an item from the cache.
	 *
	 * @param key The key of the item to be retrieved.
	 * @return the cached object or <tt>null</tt>
	 * @throws CacheException Indicates a problem accessing the item or region.
	 */
	public Object get(Object key) throws CacheException;

	/**
	 * Clear the contents of this particular cache region.
	 *
	 * @throws CacheException Indicates problem clearing the contents
	 */
	public void clear() throws CacheException;

	/**
	 * The "end state" contract of the region's lifecycle.  Called
	 * during {@link org.hibernate.SessionFactory#close()} to give
	 * the region a chance to cleanup.
	 *
	 * @throws CacheException Indicates problem shutting down
	 */
	public void destroy() throws CacheException;

	/**
	 * If the underlying cache is clustered then lock the entire region;
	 * otherwise, noop.  Generally speaking this is called prior to attempts
	 * to clear the regions contents in a concurrency-safe manner.
	 *
	 * @throws CacheException Indicates problems locking the region.
	 */
	public void lock() throws CacheException;

	/**
	 * If this is a clustered cache, unlock the region.  Always called in
	 * tandem with {@link #lock()} after the region modifications are
	 * complete.
	 *
	 * @throws CacheException Indicates problems unlocking the item.
	 */
	public void unlock() throws CacheException;

	/**
	 * If the underlying cache is clustered then lock the specified content
	 * item; otherwise, noop.  Typically this method is called prior to attempts
	 * to modify existing content within the region.
	 *
	 * @param key The cache key of the item to be locked.
	 * @throws CacheException Indicates problems locking the item.
	 */
	public void lock(Object key) throws CacheException;

	/**
	 * If this is a clustered cache, unlock the item.  Always called in
	 * tandem with {@link #lock(Object)} after the content modification is
	 * complete.
	 *
	 * @param key The cache key of the item to be unlocked.
	 * @throws CacheException Indicates problems unlocking the item.
	 */
	public void unlock(Object key) throws CacheException;

	/**
	 * The number of bytes is this cache region currently consuming in memory.
	 *
	 * @return The number of bytes consumed by this region; -1 if unknown or
	 * unsupported.
	 */
	public long getSizeInMemory();

	/**
	 * The count of entries currently contained in the regions in-memory store.
	 *
	 * @return The count of entries in memory; -1 if unknown or unsupported.
	 */
	public long getElementCountInMemory();

	/**
	 * The count of entries currently contained in the regions disk store.
	 *
	 * @return The count of entries on disk; -1 if unknown or unsupported.
	 */
	public long getElementCountOnDisk();

	/**
	 * Get the contents of this region as a map.
	 * <p/>
	 * Implementors which do not support this notion
	 * should simply return an empty map.
	 *
	 * @return The content map.
	 */
	public Map toMap();

	public long nextTimestamp();
	public int getTimeout();
}
