package org.hibernate.cache.impl;

import java.util.Properties;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.hibernate.cache.SecondLevelCache;
import org.hibernate.cache.CacheProvider;
import org.hibernate.cache.CacheException;
import org.hibernate.cache.EntityRegion;
import org.hibernate.cache.CollectionRegion;
import org.hibernate.cache.QueryResultsRegion;
import org.hibernate.cache.NoCacheProvider;
import org.hibernate.cache.TimestampsRegion;
import org.hibernate.cache.Region;
import org.hibernate.cache.Cache;
import org.hibernate.cache.OptimisticCache;
import org.hibernate.cache.OptimisticCacheSource;
import org.hibernate.util.PropertiesHelper;
import org.hibernate.util.ReflectHelper;
import org.hibernate.cfg.Environment;

/**
 * Acts as a bridge between the {@link SecondLevelCache} contract and the older
 * {@link CacheProvider} contract.
 *
 * @author Steve Ebersole
 */
public class SecondLevelCacheBridge implements SecondLevelCache {
	public static final String DEF_PROVIDER = NoCacheProvider.class.getName();
	private static final Log log = LogFactory.getLog( SecondLevelCacheBridge.class );

	private CacheProvider cacheProvider;

	public void start(Properties properties) throws CacheException {
		String providerClassName = PropertiesHelper.getString( Environment.CACHE_PROVIDER, properties, DEF_PROVIDER );
		log.info( "Cache provider: " + providerClassName );
		try {
			cacheProvider = ( CacheProvider ) ReflectHelper.classForName( providerClassName ).newInstance();
		}
		catch ( Exception cnfe ) {
			throw new CacheException( "could not instantiate CacheProvider [" + providerClassName + "]", cnfe );
		}
		cacheProvider.start( properties );
	}

	public void stop() {
		cacheProvider.stop();
		cacheProvider = null;
	}

	public boolean isMinimalPutsEnabledByDefault() {
		return cacheProvider.isMinimalPutsEnabledByDefault();
	}

	public long nextTimestamp() {
		return cacheProvider.nextTimestamp();
	}

	public EntityRegion buildEntityRegion(String regionName, Properties properties) throws CacheException {
		return new EntityRegionAdapter( cacheProvider.buildCache( regionName, properties ) );
	}

	public CollectionRegion buildCollectionRegion(String regionName, Properties properties) throws CacheException {
		return new CollectionRegionAdapter( cacheProvider.buildCache( regionName, properties ) );
	}

	public QueryResultsRegion buildQueryResultsRegion(String regionName, Properties properties) throws CacheException {
		return new QueryResultsRegionAdapter( cacheProvider.buildCache( regionName, properties ) );
	}

	public TimestampsRegion buildTimestampsRegion(String regionName, Properties properties) throws CacheException {
		return null;
	}

	public static abstract class BaseRegionAdapter implements Region {
		protected final Cache underlyingCache;

		protected BaseRegionAdapter(Cache underlyingCache) {
			this.underlyingCache = underlyingCache;
		}

		public String getName() {
			return underlyingCache.getRegionName();
		}

		public Object get(Object key) throws CacheException {
			return underlyingCache.get( key );
		}

		public void clear() throws CacheException {
			underlyingCache.clear();
		}

		public void destroy() throws CacheException {
			underlyingCache.destroy();
		}

		public void lock() throws CacheException {
			// impl note: unfortunately this was not a capability of the original Cache contract
		}

		public void unlock() throws CacheException {
			// impl note: unfortunately this was not a capability of the original Cache contract
		}

		public void lock(Object key) throws CacheException {
			underlyingCache.lock( key );
		}

		public void unlock(Object key) throws CacheException {
			underlyingCache.unlock( key );
		}

		public long getSizeInMemory() {
			return underlyingCache.getSizeInMemory();
		}

		public long getElementCountInMemory() {
			return underlyingCache.getElementCountInMemory();
		}

		public long getElementCountOnDisk() {
			return underlyingCache.getElementCountOnDisk();
		}

		public Map toMap() {
			return underlyingCache.toMap();
		}

		public long nextTimestamp() {
			return underlyingCache.nextTimestamp();
		}

		public int getTimeout() {
			return underlyingCache.getTimeout();
		}
	}

	public static class EntityRegionAdapter extends BaseRegionAdapter implements EntityRegion {
		protected EntityRegionAdapter(Cache underlyingCache) {
			super( underlyingCache );
		}

		public void intialize(OptimisticCacheSource source) {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).setSource( source );
			}
		}

		public Object read(Object key) throws CacheException {
			return underlyingCache.read( key );
		}

		public void writeLoad(Object key, Object value, Object version) {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).writeLoad( key, value, version );
			}
			else {
				underlyingCache.put( key, value );
			}
		}

		public void writeInsert(Object key, Object value, Object version) {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).writeInsert( key, value, version );
			}
			else {
				underlyingCache.update( key, value );
			}
		}

		public void writeUpdate(Object key, Object value, Object currentVersion, Object previousVersion) {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).writeUpdate( key, value, currentVersion, previousVersion );
			}
			else {
				underlyingCache.update( key, value );
			}
		}

		public void writeDelete(Object key) {
			underlyingCache.remove( key );
		}
	}

	public static class CollectionRegionAdapter extends BaseRegionAdapter implements CollectionRegion {
		protected CollectionRegionAdapter(Cache underlyingCache) {
			super( underlyingCache );
		}

		public void intialize(OptimisticCacheSource source) {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).setSource( source );
			}
		}

		public void writeLoad(Object key, Object value, Object version) throws CacheException {
			if ( underlyingCache instanceof OptimisticCache ) {
				( ( OptimisticCache ) underlyingCache ).writeLoad( key, value, version );
			}
			else {
				underlyingCache.put( key, value );
			}
		}

		public void remove(Object key) throws CacheException {
			underlyingCache.remove( key );
		}
	}

	public static class QueryResultsRegionAdapter extends BaseRegionAdapter implements QueryResultsRegion {
		protected QueryResultsRegionAdapter(Cache underlyingCache) {
			super( underlyingCache );
		}

		public void put(Object key, Object value) throws CacheException {
			underlyingCache.put( key, value );
		}

		public void remove(Object key) throws CacheException {
			underlyingCache.remove( key );
		}
	}

	public static class TimestampsRegionAdapter extends BaseRegionAdapter implements TimestampsRegion {
		protected TimestampsRegionAdapter(Cache underlyingCache) {
			super( underlyingCache );
		}

		public void write(Object key, Object value) throws CacheException {
			underlyingCache.put( key, value );
		}
	}
}
